//
//  Pearl.swift
//  Pearl
//
//  Created by Mark Meretzky on 10/8/14.
//  Copyright (c) 2014 Mark Meretzky. All rights reserved.
//

import UIKit

class Pearl {
	//Error message in Xcode 6.1: "Class variables not yet supported".
	/* class */ let stiffness: CGFloat =  0.2;	//elasticity
	/* class */ let gravity:   CGFloat =  3;
	/* class */ let mass:      CGFloat =  2;	//must be positive
	/* class */ let damping:   CGFloat =  0.7;
	/* class */ let radius:    CGFloat = 20;	//in pixels

	var location: CGPoint = CGPointZero;
	var velocity: CGPoint = CGPointZero;

	//Pull the Pearl towards point p.

	func pullTowards(p: CGPoint) {

		let force: CGPoint = CGPointMake(
			(p.x - location.x) * stiffness,
			(p.y - location.y) * stiffness + gravity * mass);

		//Newton's second law of motion: F = ma
		//Therefore a = F/m

		let acceleration: CGPoint = CGPointMake(
			force.x / mass,
			force.y / mass);

		/*
		The new velocity is the existing velocity plus the acceleration.
		Of course, in this imperfect world, a certain percentage is skimmed
		off the top by friction (damping).
		*/
		velocity = CGPointMake(
			damping * (velocity.x + acceleration.x),
			damping * (velocity.y + acceleration.y));

		/*
		The new location is the existing location plus the distance covered
		since the last call to drag.  This distance depends on the velocity.
		In fact, it is the velocity.
		*/
		location = CGPointMake(
			location.x + velocity.x,
			location.y + velocity.y);

		let rect: CGRect = CGRectMake(
			location.x - radius,
			location.y - radius,
			2 * radius,
			2 * radius);

		//It's okay to use CGContextRef here, because this method is called
		//by the View's drawRect(_:) method.
		let c: CGContextRef = UIGraphicsGetCurrentContext();

		//Draw the circle.
		CGContextBeginPath(c);
		CGContextAddEllipseInRect(c, rect);
		CGContextSetRGBFillColor(c, 1, 1, 1, 1);	//white, opaque
		CGContextFillPath(c);

		//Draw the straight line.
		CGContextMoveToPoint(c, location.x, location.y);
		CGContextAddLineToPoint(c, p.x, p.y);
		CGContextSetRGBStrokeColor(c, 1, 1, 1, 1);
		CGContextStrokePath(c);
	}

}
